open Godi_package

exception SysError of string

let file_exists f =
    try
        Unix.access f [Unix.F_OK]; true
    with Unix.Unix_error (_,_,_) -> false

let recursive_unlink dirname =
    if Sys.command ("rm -rf " ^ dirname) <> 0 then
        raise (SysError ("Unable to recursively remove " ^ dirname))
    else ()

(* note: copy needs to be smart if second arg is a directory *)
let copy f1 f2 =
    if Sys.command ("cp " ^ f1 ^ " " ^ f2) <> 0 then
        raise (SysError ("Unable to copy: " ^ f1 ^ " --> " ^ f2))
    else ()

(* here, the second arg is always a dir *)
let copydir d1 d2 =
    if Sys.command ("cp -R " ^ d1 ^ " " ^ d2) <> 0 then
        raise (SysError ("Unable to copydir: " ^ d1 ^ " --> " ^ d2))
    else ()

let write_file str filename =
    let chan = open_out filename in
    output_string chan str;
    close_out chan

let write_file_opt stro filename =
    match stro with
        None -> ()
      | Some str -> write_file str filename

let makefile gp =
    let version = gp.version in
    let pkgname = gp.pkgname in
    let revision = gp.revision in
    let distname = gp.distname in
    let distfiles = gp.distfiles in
    let categories = gp.categories in
    let master_sites = gp.master_sites in
    let maintainer = gp.maintainer in
    let homepage = gp.homepage in
    let comment = gp.comment in
    let confopt_defaults = gp.confopt_defaults in
    let pkg = gp.pkg in
    let addl_make_flags = gp.addl_make_flags in
    let depends = gp.depends in
    let build_depends = gp.build_depends in
    let has_configure = gp.has_configure in
    let use_gmake = gp.use_gmake in
    let all_target = gp.all_target in
    let all_target_opt = gp.all_target_opt in
    let docfiles = gp.docfiles in

    interpolate file "BSDMakefile.template"

let f ~godi_localbase ~godi_package ~replace =
    let gp = godi_package in

    (* sanity check *)
    if not (file_exists (godi_localbase ^ "/etc/godi.conf")) then
        raise (SysError
            (godi_localbase ^ " does not look like a godi installation."));

    (* make the directory *)
    let inst_dir = godi_localbase ^ "/build/" ^ gp.makefile.categories
                                        ^ "/" ^ gp.makefile.pkg in

    (* check to see if the directory already exists *)
    (if file_exists inst_dir then
        if replace then
            recursive_unlink inst_dir
        else
          raise (SysError ("Can't write to " ^ inst_dir ^ ": file exists.")));

    (* mkdirs *)
    let filesdir = inst_dir ^ "/files" in
    let patchesdir = inst_dir ^ "/patches" in
    Unix.mkdir inst_dir 0o755;
    Unix.mkdir patchesdir 0o755;

    (* write files *)
    write_file (makefile gp.makefile) (inst_dir ^ "/Makefile");
    write_file gp.descr (inst_dir ^ "/DESCR");
    write_file_opt gp.buildmsg (inst_dir ^ "/BUILDMSG");
    write_file_opt gp.confopts (inst_dir ^ "/CONFOPTS");

    (* copy in patches & files *)
    List.iter (fun patchfile -> copy patchfile patchesdir) gp.patches;
    (match gp.filesdir with
        None -> ()
      | Some fdir -> copydir fdir inst_dir);

    (* generate checksums *)
    let bmake = godi_localbase ^ "/bin/bmake" in
    if Sys.command ("cd " ^ inst_dir ^ " && " ^ bmake ^ " makesum") <> 0 then
        raise (SysError ("Error generating checksum."));
    if Sys.command ("cd " ^ inst_dir ^ " && " ^ bmake ^ " makepatchsum") <> 0
    then
        raise (SysError ("Error generating patch checksums."));

    (* copy in the spec file *)
    copy gp.specfile inst_dir;

    print_endline ("Package written to " ^ inst_dir ^ ".")
